#include "_afintrn.h"

#define QUALITY 30 /* nr of spline-sections to calculate per b-spline */
#define COV_SUB 4 /* subdivison for antialiasing */

/*
 *  PIXINFO struct stores information on
 *  the intersections of a spline with a pixel
 *  they are stores in lined lists per scabline
 */
typedef struct PIXINFO
{
   double xin,yin,xuit,yuit; /* entrance and exit coordinates */
   int pix;                  /* pixel coordinate in scanline */
   int start, end;           /* used to mark start/endpoints */
   struct PIXINFO *next;     /* double linked list handles */
   struct PIXINFO *last;
} PIXINFO;


/*
 * put the pointer 'list' back to the beginning of the linked list
 * after elements have been added before the first element
 */
static void rewind_list(PIXINFO **list)
{
   PIXINFO *l = *list;

   while(l->last)
     l = l->last;

   *list = l;
   
}

/* destroys a list, freeing all objects in it */
static void destroy_list(PIXINFO *l)
{
   PIXINFO *last;
   while(l)
   {
      last = l;
      l = l->next;
      free(last);
   }
}


/*
 * deletes form the list (does not free object)
 */
static void delete_from_list(PIXINFO **list, PIXINFO *p)
{

   //! check of p  in list zit
   if (p == *list)
      *list = p->next;

   if (p->next)
      p->next->last = p->last;

   if (p->last)
      p->last->next = p->next;


   p->next = p->last = NULL;
}



/*
 * inserts p in the list, sorted on pix/xin/yin
 */
static void insert_in_list(PIXINFO **list, PIXINFO *p)
{
   PIXINFO tmp;
   PIXINFO *last;
   PIXINFO *l = *list;

   tmp.last = NULL;
   tmp.next = *list;


   last = &tmp;

   text_mode(0);

   while(l
         && (l->pix < p->pix ||
             (
              l->pix == p->pix
              && (l->xin - p->xin < -0.005 // list->xin <= p->xin
                 || (
                     (l->xin - p->xin > -0.005)
                     && (l->xin - p->xin < 0.005)
                     && l->yin - p->yin < -0.005 // list->yin < p->yin
                    )
                 )
             )
            )
        )
   {
      last = l;
      l = l->next;
   }

   p->next = last->next;

   if (last != &tmp)
       p->last = last;
   else
       p->last = NULL;

   if (p->next)
       p->next->last = p;

   if (p->last)
       p->last->next = p;

   rewind_list(list);
}

/*
 * adds p to the end of the list
 */
static void add_to_list(PIXINFO **list, PIXINFO *p)
{
   PIXINFO *l = *list;

   if (!l)
   {
      *list = p;
//      p->last = p->next = NULL;
      return;
   }

   while(l->next)
      l = l->next;

   l->next = p;
   p->last = l;
   p->next = NULL;
   
}



/*
 * calculates the intersections with the pixels of the given spline
 * storing a list of pixelintersections in the given array
 * possibly adding to existing lists in the array
 * the array should be initialised to 0
 * the true scale is *0.01 since points are integers
 * and we want sub-pixel accurcay also at scale 1
 */
static void spline_2_scanlist(double scale_x, double scale_y,int height, PIXINFO **res, int _points[8], int splinenr)
{
   int x[QUALITY] , y[QUALITY]; /* to store the result of calc_spline */
   int xlastcross,ylastcross;   /* last detected pixelboundary crossing */
   int start;                   /* is the a line start ? */
   int xpix, ypix;              /* pos in pixel coords of last stored pixel */
   int cxpix = 0, cypix = 0;            /* pos in pixel coords of current point */
   int mx,my;                   /* mean between last/current point  */
   int i,j;
   
   int subdivide;       /* number of subdivisions of a straight linepiece to prevent skipping pixels */
   int len;             /* length of a linepiece */
   int dx,dy;           /* dx/dy of a linepiece */
   int cx,cy,lx,ly;     /* current x/y, last x/y (shape coords (=100*pix at scale 1) */
   
   PIXINFO *p;

   int points[8];       /* control points of current spline, scaled */

   for (i=0;i<8;i+=2)
   {
      points[i] = (int)(_points[i] * scale_x);
   }

   for (i=1;i<8;i+=2)
   {
      points[i] = (int)(_points[i] * scale_y);
   }
   
   /* call allegro calc-spline func */
   calc_spline(points, QUALITY ,x ,y);



   /* set last boundary crossing to spline start
    * this is ofcourse not a real boundary crossing
    * but we'll also mark this as a linestart
    * so we can later match up
    * with an endpoint of another spline
    */
   xlastcross = x[0];
   ylastcross = y[0];

   xpix = floor(x[0] / 100.0);
   ypix = floor(y[0] / 100.0);

   
   start = splinenr;
   
   lx = x[0];
   ly = y[0];

   /* walk all calculated spline points */
   for (i=1;i<QUALITY;i++)
   {
      dx = (x[i] - x[i-1]);
      dy = (y[i] - y[i-1]);
      len = (int)sqrt(dx*dx  + dy*dy);

      subdivide = len /20 + 1;

      /* subdivide to prevent skipping of pixels at large scales */
      for (j=0;j<=subdivide;j++) //  <= is goed, moet om te zorgen dat eindpunten precies op randen goed gaan
      {
         cx = x[i-1] + dx * j / subdivide;
         cy = y[i-1] + dy * j / subdivide;

         /* determine current pixel position */
         cxpix = floor(cx / 100.0);
         cypix = floor(cy / 100.0);
         
         if (xpix != cxpix || ypix != cypix) /* crossed pixel border */
         {
            mx = (cx + lx) / 2;
            my = (cy + ly) / 2;
   
            if (ypix != cypix) /* y changed */
            {
               /* force my to pixel boundary
                * for simplicity reasons I do not
                * truly calculate the intersection
                * just push the point to the boundary
                */
               if (my >=0)
                  my = (my+50) / 100;
               else
                  my = (my-50) / 100;
                  
               my *= 100;
            }
   
            if (xpix != cxpix) /* x changed */
            {
               /* force mx to pixel boundary */
               if (mx >= 0)
                  mx = (mx+50) / 100;
               else
                  mx = (mx-50) / 100;
                  
               mx *= 100;
            }
   
            /* check if we are within the range of scanlines requested */
            if (ypix >=0 && ypix < height)
            {
               /* build a new PIXINFO object */
               p = malloc(sizeof(PIXINFO));

               //! needs malloc check
               ASSERT(p);
               
               p->next = p->last = NULL;
               p->start = start;
               p->end = -1;
               p->xin = xlastcross/100.0 - xpix;
               p->yin = ylastcross/100.0 - ypix;
               p->xuit = mx/100.0 - xpix;
               p->yuit = my/100.0 - ypix;
               p->pix = xpix;
   
   
                  
               if (!res[ypix])
               {
                  /* nothing yet in this scanline
                   * this is the first entry
                   */
                  res[ypix] = p;
               }
               else
               {
                  insert_in_list(&(res[ypix]), p);
               }
            }
            /* ok, setup everything for the next pixel now we're
             * finished with the current one
             */
            start = -1; /* since we crossed the border the next pixel cannot be the startpoint */
            xpix = cxpix;
            ypix = cypix;
            xlastcross = mx;
            ylastcross = my;
         }
         lx = cx;
         ly = cy;
      }
   }

   /* now do the endpoint */
   if (cypix >=0 && cypix < height)
   {
      p = malloc(sizeof(PIXINFO));

      //! needs malloc check
      ASSERT(p);

      /* same as above */
      p->next = p->last = NULL;
      p->start = start;
      p->end = splinenr;
      p->xin = xlastcross/100.0 - cxpix;
      p->yin = ylastcross/100.0 - cypix;
      p->xuit = x[QUALITY-1]/100.0 - cxpix;
      p->yuit = y[QUALITY-1]/100.0 - cypix;
      p->pix = cxpix;
   
      if (!res[cypix])
      {
         res[cypix] = p;
      }
      else
      {
         insert_in_list(&(res[cypix]), p);
      }

      
   }

   
}

static void line_2_scanlist(double scale_x, double scale_y,int height, PIXINFO **res, int _points[4], int linenr)
{
   int x[2] , y[2]; /* to store the result of calc_spline */
   int xlastcross,ylastcross;   /* last detected pixelboundary crossing */
   int start;                   /* is the a line start ? */
   int xpix, ypix;              /* pos in pixel coords of last stored pixel */
   int cxpix = 0, cypix = 0;            /* pos in pixel coords of current point */
   int mx,my;                   /* mean between last/current point  */
   int i,j;
   
   int subdivide;       /* number of subdivisions of a straight linepiece to prevent skipping pixels */
   int len;             /* length of a linepiece */
   int dx,dy;           /* dx/dy of a linepiece */
   int cx,cy,lx,ly;     /* current x/y, last x/y (shape coords (=100*pix at scale 1) */
   
   PIXINFO *p;


   for (i=0;i<2;i++)
   {
      x[i] = (int)(_points[2*i] * scale_x);
      y[i] = (int)(_points[2*i+1] * scale_x);
   }

   

   /* set last boundary crossing to spline start
    * this is ofcourse not a real boundary crossing
    * but we'll also mark this as a linestart
    * so we can later match up
    * with an endpoint of another spline
    */
   xlastcross = x[0];
   ylastcross = y[0];

   xpix = floor(x[0] / 100.0);
   ypix = floor(y[0] / 100.0);

   
   start = linenr;
   
   lx = x[0];
   ly = y[0];

   dx = (x[1] - x[0]);
   dy = (y[1] - y[0]);
   len = (int)sqrt(dx*dx  + dy*dy);

   subdivide = len /20 + 1;

   /* subdivide to prevent skipping of pixels at large scales */
   for (j=0;j<=subdivide;j++) //  <= is goed, moet om te zorgen dat eindpunten precies op randen goed gaan
   {
      cx = x[0] + dx * j / subdivide;
      cy = y[0] + dy * j / subdivide;

      /* determine current pixel position */
      cxpix = floor(cx / 100.0);
      cypix = floor(cy / 100.0);
         
      if (xpix != cxpix || ypix != cypix) /* crossed pixel border */
      {
         mx = (cx + lx) / 2;
         my = (cy + ly) / 2;
 
         if (ypix != cypix) /* y changed */
         {
               /* force my to pixel boundary
                * for simplicity reasons I do not
                * truly calculate the intersection
                * just push the point to the boundary
                */
            if (my >=0)
               my = (my+50) / 100;
            else
               my = (my-50) / 100;
               
            my *= 100;
         }

         if (xpix != cxpix) /* x changed */
         {
            /* force mx to pixel boundary */
            if (mx >= 0)
               mx = (mx+50) / 100;
            else
               mx = (mx-50) / 100;
               
            mx *= 100;
         }

         /* check if we are within the range of scanlines requested */
         if (ypix >=0 && ypix < height)
         {
            /* build a new PIXINFO object */
            p = malloc(sizeof(PIXINFO));

            //! needs malloc check
            ASSERT(p);
            
            p->next = p->last = NULL;
            p->start = start;
            p->end = -1;
            p->xin = xlastcross/100.0 - xpix;
            p->yin = ylastcross/100.0 - ypix;
            p->xuit = mx/100.0 - xpix;
            p->yuit = my/100.0 - ypix;
            p->pix = xpix;


//            circlefill(screen, 400 * (p->xin+xpix) / COV_SUB,400 * (p->yin+ypix) / COV_SUB,3, makecol(255,0,0));
//            circlefill(screen, 400 * (p->xuit+xpix) / COV_SUB,400 * (p->yuit+ypix) / COV_SUB,3, makecol(255,0,255));

            if (!res[ypix])
            {
               /* nothing yet in this scanline
                * this is the first entry
                */
               res[ypix] = p;
            }
            else
            {
               insert_in_list(&(res[ypix]), p);
            }
         }
         /* ok, setup everything for the next pixel now we're
          * finished with the current one
          */
         start = -1; /* since we crossed the border the next pixel cannot be the startpoint */
         xpix = cxpix;
         ypix = cypix;
         xlastcross = mx;
         ylastcross = my;
      }
      lx = cx;
      ly = cy;
   }

   /* now do the endpoint */
   if (cypix >=0 && cypix < height)
   {
      p = malloc(sizeof(PIXINFO));

      //! needs malloc check
      ASSERT(p);

      /* same as above */
      p->next = p->last = NULL;
      p->start = start;
      p->end = linenr;
      p->xin = xlastcross/100.0 - cxpix;
      p->yin = ylastcross/100.0 - cypix;
      p->xuit = x[1]/100.0 - cxpix;
      p->yuit = y[1]/100.0 - cypix;
      p->pix = cxpix;
   
      if (!res[cypix])
      {
         res[cypix] = p;
      }
      else
      {
         insert_in_list(&(res[cypix]), p);
      }

      
   }

   
}


/*
 * searches through the list for endpoints/startpoints
 * and tries to find the corresponding end/startpoints
 * it comines the two and removes the second occurence from the
 * list. since this can never remove the first element
 * of the list is doesn't need a PIXINFO **, but a PIXINFO * suffices
 */
static void tie_ends(PIXINFO *list)
{
   PIXINFO *l, *k;
   int found;

   l = list;
   while(l)
   {
      if (l->end>=0) /* found an endpoint, find matching startpoint */
      {
         
         k = l->next;
         found = FALSE;
         while(k && k->pix == l->pix)
         {
            if (k->start >=0
                && abs(k->xin - l->xuit) < 0.01
                && abs(k->yin - l->yuit) < 0.01
               )
            {  /* found it */
               delete_from_list(&list, k);
               l->xuit = k->xuit;
               l->yuit = k->yuit;
               l->end = k->end; // k can be within the same pixel
               free(k);
               found = TRUE;
               break;
            }
            k = k->next;
            
         }
         if (!found)
         {
            /* complain */
            TRACE("draw_shape1: curve not closed: %d,  %g,%g, %g,%g start %d end %d\n"
            ,l->pix,l->xin,l->yin,l->xuit,l->yuit,l->start,l->end);
            l->end = -1;   // mark it as done anyway, to let the program go on
         }
         
      }
      
      if (l->start >=0) /* found startpoint search matching endpoint */
      {
         k = l->next;
         found = FALSE;
         while(k && k->pix == l->pix)
         {
            if (k->end >=0
                && abs(l->xin - k->xuit) < 0.01
                && abs(l->yin - k->yuit) < 0.01
               )
            {  /* found it */
               delete_from_list(&list, k);
               l->xin = k->xin;
               l->yin = k->yin;
               l->start = k->start;
               free(k);
               found = TRUE;
               break;
            }
            k = k->next;
         }
         if (!found)
         {
            TRACE("draw_shape2: curve not closed: %d,  %g,%g, %g,%g start %d end %d\n"
            ,l->pix,l->xin,l->yin,l->xuit,l->yuit,l->start,l->end);
            l->start = -1; // mark it as done anyway, to let the program go on
         }
         
      }

      if (l->end<0 && l->start<0)
         l = l->next;
   }
   
}

static void calc_scanline(PIXINFO **list, int w, unsigned char *store, int antialias);

/* helperfunction for coverage() */
static int nr_lines_along_left_edge(int pixel, PIXINFO *l)
{
   int count = 0;
   while(l)
   {
      if (l->pix == pixel)
      {
         if (l->xin < .005)
            count++;
            
         if (l->xuit < .005)
            count++;
      }
      l = l->next;
   }

   return count;
}

/* helperfunction for coverage() */
static int nr_ulcorner_points(int pixel, PIXINFO *l)
{
   int count = 0;
   while(l)
   {
      if (l->pix == pixel)
      {
         if (l->xin < .005 && l->yin < .005)
            count++;
            
         if (l->xuit < .005 && l->yuit < .005)
            count++;
      }
      l = l->next;
   }

   return count;
}


/* returns coverage percentage for a pixel
 * given a linked list of PIXINFO's
 * this is done by subdividing the pixel in
 * 10x10 sub pixels and determining which
 * sub-pixels are black
 */
static int coverage(PIXINFO *list)
{
   static PIXINFO *pif[COV_SUB];
   PIXINFO *l;
   int i,j;
   int pts[4];
   unsigned char scanline[COV_SUB];
   int ret = 0;
   int pix = list->pix;
   int nl;
   
    /* color of ul corner taken white, then alternating at
     * each start/endpoint along the left edge (down)
     * this determines the startcolor of each scanline
     */
   int firstblack = FALSE;
   int ncp;


   ncp = nr_ulcorner_points(pix, list) % 2;

   if (ncp)
      firstblack =TRUE;

   for (i = 0;i<COV_SUB;i++)
   {
      pif[i] = NULL;
   }

/*   rectfill(screen,0,0,400,400, makecol(255,255,255));
 *  for (i=0;i<COV_SUB;i++)
 *      for (j=0;j<COV_SUB;j++)
 *          rect(screen, 400 * j / COV_SUB,400 * i / COV_SUB,400 * (j+1) / COV_SUB,400 * (i+1) / COV_SUB, makecol(255,0,0));
 */
   /* first create scanline lists from the linepieces */
   for(l = list;l;l=l->next)
   {
      ASSERT(pix == l->pix);
      pts[0] = l->xin * 100;
      pts[1] = l->yin * 100;
      pts[2] = l->xuit * 100;
      pts[3] = l->yuit * 100;

      line_2_scanlist(COV_SUB, COV_SUB, COV_SUB, pif, pts, 0);
/* line(screen, 4 * pts[0], 4 * pts[1], 4 * pts[2], 4 * pts[3], makecol(255,0,255));
 */
   }



   for (i=0;i<COV_SUB;i++)
   {
       nl = nr_lines_along_left_edge(0,pif[i]) % 2;
       ncp = nr_ulcorner_points(0, pif[i]) % 2;

       
       calc_scanline(&(pif[i]), COV_SUB, scanline, FALSE);


       if (ncp)  /* flip for just this row */
          firstblack = !firstblack;

       // count black xub-pixels in scanline
       for (j=0;j<COV_SUB;j++)
       {
           if (firstblack)
           {
             if ((scanline[j]))
             {
/*                rect(screen, 400 * j / COV_SUB+1,400 * i / COV_SUB+1,400 * (j+1) / COV_SUB-1,400 * (i+1) / COV_SUB-1, makecol(0,100,0));
 */
                ret++;
             }
           }
           else
           {
             if (!(scanline[j]))
             {
/*              rect(screen, 400 * j / COV_SUB+1,400 * i / COV_SUB+1,400 * (j+1) / COV_SUB-1,400 * (i+1) / COV_SUB-1, makecol(0,100,0));
 */
                ret++;
             }
           }
       }
       destroy_list(pif[i]);

       if (ncp)
          firstblack = !firstblack;
       
       if (nl) // flip startcolor for bext line  if we have an uneven nr of lines on the left edge
          firstblack = ! firstblack;
       
   }

//   textprintf(screen, font,0,0,makecol(0,0,0),"%d pix %d",ret,list->pix);
//   readkey();
   return ret;
}

/* calc_scanline
 * this takes a list of PIXINFO's and a char buffer the width of
 * the image to render and fills the char buffer
 * this changes the list, so you need to pass a refernce to it
 */
static void calc_scanline(PIXINFO **scanlist, int w, unsigned char *store, int antialias)
{



/* PASS one: determine color (b or w) of upper left corner
 * of each pixel in the scanline
 */

   /* first walk the list till we come to pixel 0
    * flipping color each time we get an yin or yuit of 0
    * to determine the color of the upper left corner
    * of pixel 0
    */
   PIXINFO *list = *scanlist;
   int cornerblack = FALSE;
   int i;
   int lx;
   int curpix;
   PIXINFO *dumplist;
   PIXINFO *sublist;
   PIXINFO *next;
   
   int cover;

   while(list && list->pix < 0)
   {
      if (list->yin < .005)
      {
         cornerblack = !cornerblack;
      }
      
      if (list->yuit < .005)
      {
         cornerblack = !cornerblack;
      }
      list = list->next;
   }
   
   if (!list)
   {
      ASSERT(!cornerblack); // if the list is done before we reached 0, the line must be white
      for (i=0; i< w;i++)
          store[i] = 255; // fully white/clear
          
      return;
   }

   lx = 0;

   if (cornerblack)
      store[lx] = 0;
   else
      store[lx] = 255;

   while(list && list->pix < w)
   {
      if (list->pix > lx)
      {
         for (i=lx+1;i<=list->pix;i++)
         {
            if (cornerblack) //! temp
               store[i] = 0;
            else
               store[i] = 255;
         }
         lx = list->pix;
      }

      if (list->yin < .01)
          cornerblack = !cornerblack;
            
      if (list->yuit < .01)
          cornerblack = !cornerblack;
          
      list = list->next;
      
   }

   for (i=lx+1;i<w;i++)
   {
      if (cornerblack)
         store[i] = 0;
      else
         store[i] = 255;
   }

   if (!antialias)
      return;

/* end of PASS ONE */

/* start antialias pass PASS TWO */
/* we can safely mess with the scanlists here , since
 * we don't need them anymore afther this */

   list = *scanlist;
   dumplist = NULL;

   while(list)
   {
      curpix = list->pix;
      sublist = list;



      while(1)
      {
         next = list->next;
      
         delete_from_list(scanlist, list);
         add_to_list(&dumplist, list);


         list = next;
         
         if (!(list && (list->pix == curpix)))
            break;
      }

      cover = 255 * coverage(sublist) / (COV_SUB * COV_SUB);


      if (curpix >=0 && curpix < w)
      {
         if (store[curpix])
         {
            store[curpix] = 255 - cover;
         }
         else
         {
            store[curpix] = cover;
         }
      }
   }

   *scanlist = dumplist;

}


/*
 * render lists
 * takes nr linked lists of PIXINFO's
 * a bitmap and x and y and a max width, and renders the scanlines in the lists
 */
static void render_lists(BITMAP *onto, int x, int y, int color,int w, int nr, PIXINFO **lists)
{
   int i,j;
   unsigned char *store = malloc(sizeof(char) * w);

   ASSERT(store); //! malloc

   for (i=0;i<nr;i++)
   {
      if (lists[i])
      {
        if (color < 0)
           calc_scanline(&(lists[i]),w, store, TRUE);
        else
           calc_scanline(&(lists[i]),w, store, FALSE);


         
         for (j=0;j<w;j++)
         {
            if (color >=0)
            {
               if (store[j]< 128)
               {
                  putpixel(onto,j+x,i+y, color);
                  //! need to add color/alpha support
               }
            }
            else
            {
                putpixel(onto,j+x,i+y, 255 - store[j]);
            }
         }
      }
   }

   free(store);
}


/*
 * draw_shape:
 * the main entrypoint of this file
 * takes a pointer to a SPLINE_SHAPE object and
 * draws it on a bitmap
 */
 //! need to add color support
void draw_shape(BITMAP *onto, int x, int y, int color, double scale_x, double scale_y, SPLINE_SHAPE *s)
{
   PIXINFO **pif ;
   int nr_scanlines = (int)(s->h * scale_y);
   int i;


   pif  = malloc(sizeof(PIXINFO *) * nr_scanlines);

   ASSERT(pif); //! malloc check + static & growing?
   

   for (i=0;i<nr_scanlines;i++)
   {
      pif[i] = NULL;
   }

   /*
    * convert all splines of the object to
    * PIXINFO lists containing spline-pixel intersection info
    */
   for (i=0;i<s->nr_splines;i++)
       spline_2_scanlist(scale_x, scale_y, nr_scanlines, pif, s->data[i], i);
       

   /*
    * tie the separate splines together to one closed path
    */
   for (i=0;i<nr_scanlines;i++)
   {
      if (pif[i])
          tie_ends(pif[i]);
   }

   /* draw the thing */
   render_lists(onto,x,y,color, (int)(s->w * scale_x),nr_scanlines,pif);

   /* cleanup*/
   for (i=0;i<nr_scanlines;i++)
   {
      if (pif[i])
         destroy_list(pif[i]);
   }

   free(pif);
}

/* draws the shape in outline mode */
void draw_shape_outline(BITMAP *onto, int x, int y, int color,double scale_x, double scale_y, SPLINE_SHAPE *s)
{
   int i,j;
   int pts[8];

   
   for (i=0;i<s->nr_splines;i++)
   {
      for (j=0;j<8;j+=2)
      {
         pts[j] = (int)(s->data[i][j] * scale_x / 100);
         pts[j] += x;
      }
   
      for (j=1;j<8;j+=2)
      {
         pts[j] = (int)(s->data[i][j] * scale_y / 100);
         pts[j] += y;
      }
      spline(onto, pts, color);
   }

}



/* some debugging functions follow */


/*
void dumplist(PIXINFO *l)
{
   while(l)
   {
      TRACE("%p: %d   %g %g, %g %g  s%d e%d\n",
            l,l->pix, l->xin,l->yin,
             l->xuit,l->yuit,l->start, l->end);
      l = l->next;
   }
}

#define SC 5

void lline(int x1,int y1, int x2, int y2, int c)
{
 line(screen, 200+x1/SC,y1/SC,200+x2/SC,y2/SC,c);
}

void ppx(int x, int y, int c)
{
 putpixel(screen,200+x/SC,y/SC,c);
}

void cc(int x, int y, int c)
{
 circlefill(screen,200+x/SC,y/SC,3,c);
}
void drawpi(PIXINFO *p, int y)
{
   lline(100*p->xin+100*p->pix,100*p->yin+100*y,100*p->xuit+100*p->pix,100*p->yuit+100*y, makecol(0,0,0));
}

void drawlist(int nr, PIXINFO **li)
{
   int i;
   PIXINFO *l;
   
   for (i = 0; i < nr; i++)
   {
      for (l = li[i];l;l=l->next)
      {
          drawpi(l,i);
      }
   }
}


*/

